Library IEEE;
Use IEEE.Std_Logic_1164.All;
Use IEEE.Numeric_Std.All;

Entity Logica Is
	Port (A     : In  Std_Logic_Vector(15 DownTo 0);
			D     : In  Std_Logic_Vector( 7 DownTo 0);
			nWR   : In  Std_Logic;
			nRD   : In  Std_Logic;
			nIORQ : In  Std_Logic;
			nMREQ : In  Std_Logic;
			nRESET: In  Std_Logic;
			nMDIS : Out Std_Logic;
			Sel   : Out Std_Logic);
End Logica;

Architecture Behavioral Of Logica Is

   Type Puertos Is Array(0 To 1)Of Std_Logic_Vector(3 DownTo 0);

   Signal P  : Puertos;					-- Puertos $80 y $81
	Signal Clk: Std_Logic;				-- Reloj para registros

Begin
   -- Reloj para registros I/O
	Clk <= '1' When (nIORQ = '0') And (nWR = '0') And (A(7 DownTo 1) = "1000000") Else '0';

   -- I/O
   Process (nRESET, Clk, A, D)
   Begin
        If nRESET = '0' Then
           P <= ("0000", "0000");	-- Igual que la original
	     ElsIf Rising_Edge(Clk) Then
		     P(To_Integer(Unsigned'("" & A(0)))) <= D(3 DownTo 0);
	     End If;
   End Process;

   -- Memoria
   Process (A, nMREQ, nRD, P)
   Variable I: Integer Range 0 To 3;
   Begin
        nMDIS <= 'Z';					-- No mapeo
	     Sel   <= '0';					-- Asigno para prevenir latch
        If (nMREQ = '0') And (nRD = '0') Then
   	     I:=To_Integer(Unsigned(A(15 DownTo 14)));
	        If P(0)(I) = '0' Then		-- Habilitamos con 0
              nMDIS <= '0';			-- Mapeo
	           Sel   <= '0';			-- Rom baja
		     ElsIf P(1)(I) = '1' Then	-- Habilitamos con 1
              nMDIS <= '0';			-- Mapeo
	           Sel   <= '1';			-- Rom alta
		     End If;
	     End If;
   End Process;

End Behavioral;